@@ -106,22 +106,21 @@ static void
 totem_screensaver_update_from_state (TotemObject *totem,
 				     TotemScreensaverPlugin *pi)
 {
-	gboolean lock_screensaver_on_audio, visual_effects, can_get_frames;
+	gboolean lock_screensaver_on_audio, can_get_frames;
 	BaconVideoWidget *bvw;
 	GConfClient *gc;
 
 	bvw = BACON_VIDEO_WIDGET (totem_get_video_widget ((Totem *)(totem)));
 	gc = gconf_client_get_default ();
 
-	visual_effects = gconf_client_get_bool (gc,
-						GCONF_PREFIX"/show_vfx",
-						NULL);
 	lock_screensaver_on_audio = gconf_client_get_bool (gc, 
 							   GCONF_PREFIX"/lock_screensaver_on_audio",
 							   NULL);
 	can_get_frames = bacon_video_widget_can_get_frames (bvw, NULL);
 
-	if (totem_is_playing (totem) != FALSE && (lock_screensaver_on_audio || can_get_frames))
+	if (totem_is_playing (totem) != FALSE && can_get_frames)
+		totem_scrsaver_disable (pi->scr);
+	else if (totem_is_playing (totem) != FALSE && !lock_screensaver_on_audio)
 		totem_scrsaver_disable (pi->scr);
 	else
 		totem_scrsaver_enable (pi->scr);
