@@ -258,21 +258,33 @@ totem_python_module_init_python (void)
 	gettext_args = Py_BuildValue ("ss", GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	PyObject_CallObject (install, gettext_args);
 	Py_DECREF (gettext_args);
+
+	/* ideally totem should clean up the python stuff again at some point,
+	 * for which we'd need to save the result of SaveThread so we can then
+	 * restore the state in a class_finalize or so, but since totem doesn't
+	 * do any clean-up at this point, we'll just skip this as well */
+	PyEval_SaveThread();
 }
 
 static gboolean
 totem_python_module_load (GTypeModule *gmodule)
 {
 	TotemPythonModulePrivate *priv = TOTEM_PYTHON_MODULE_GET_PRIVATE (gmodule);
+	PyGILState_STATE state;
 	PyObject *main_module, *main_locals, *locals, *key, *value;
 	PyObject *module, *fromlist;
 	Py_ssize_t pos = 0;
+	gboolean res = FALSE;
+
+	g_return_val_if_fail (Py_IsInitialized (), FALSE);
+
+	state = pyg_gil_state_ensure();
 
 	main_module = PyImport_AddModule ("__main__");
 	if (main_module == NULL)
 	{
 		g_warning ("Could not get __main__.");
-		return FALSE;
+		goto done;
 	}
 
 	/* If we have a special path, we register it */
@@ -296,7 +308,7 @@ totem_python_module_load (GTypeModule *gmodule)
 	Py_DECREF (fromlist);
 	if (!module) {
 		PyErr_Print ();
-		return FALSE;
+		goto done;
 	}
 
 	locals = PyModule_GetDict (module);
@@ -308,13 +320,17 @@ totem_python_module_load (GTypeModule *gmodule)
 		if (PyObject_IsSubclass (value, (PyObject*) PyTotemPlugin_Type))
 		{
 			priv->type = totem_python_object_get_type (gmodule, value);
-			return TRUE;
+			res = TRUE;
+			goto done;
 		}
 	}
 
-	g_debug ("Failed to find any totem.Plugin-derived classes in Python plugin");
+	g_warning ("Failed to find any totem.Plugin-derived classes in Python plugin");
 
-	return FALSE;
+done:
+
+	pyg_gil_state_release (state);
+	return res;
 }
 
 static void
@@ -367,10 +383,13 @@ static void
 totem_python_module_finalize (GObject *object)
 {
 	TotemPythonModulePrivate *priv = TOTEM_PYTHON_MODULE_GET_PRIVATE (object);
-	g_debug ("Finalizing Python module %s", g_type_name (priv->type));
 
-	g_free (priv->module);
-	g_free (priv->path);
+	if (priv && priv->type) {
+		g_debug ("Finalizing Python module %s", g_type_name (priv->type));
+
+		g_free (priv->module);
+		g_free (priv->path);
+	}
 
 	G_OBJECT_CLASS (totem_python_module_parent_class)->finalize (object);
 }
@@ -485,7 +504,7 @@ run_gc (gpointer data)
 #endif
 
 void
-totem_python_garbage_collect ()
+totem_python_garbage_collect (void)
 {
 #if 0
 	if (Py_IsInitialized() && idle_garbage_collect_id == 0) {
@@ -510,7 +529,7 @@ finalise_collect_cb (gpointer data)
 #endif
 
 void
-totem_python_shutdown ()
+totem_python_shutdown (void)
 {
 #if 0
 	if (Py_IsInitialized ()) {
@@ -523,7 +542,7 @@ totem_python_shutdown ()
 			/* loop */;
 
 		/* This helps to force Python to give up its shell reference */
-		g_timeout_add (1000, finalise_collect_cb, NULL);
+		g_idle_add (finalise_collect_cb, NULL);
 
 		/* Disable for now, due to bug 334188
 		Py_Finalize ();*/
