@@ -131,6 +131,7 @@ enum {
 enum {
 	PLAYING_COL,
 	FILENAME_COL,
+	FILENAME_ESCAPED_COL,
 	URI_COL,
 	TITLE_CUSTOM_COL,
 	SUBTITLE_URI_COL,
@@ -1631,7 +1632,7 @@ totem_playlist_init (TotemPlaylist *playlist)
 
 	/* tooltips */
 	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(playlist->priv->treeview),
-					 FILENAME_COL);
+					 FILENAME_ESCAPED_COL);
 
 	/* The configuration */
 	init_config (playlist);
@@ -1654,12 +1655,13 @@ totem_playlist_new (void)
 }
 
 static gboolean
-totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
-		const char *display_name)
+totem_playlist_add_one_mrl (TotemPlaylist *playlist,
+			    const char *mrl,
+			    const char *display_name)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
-	char *filename_for_display, *uri;
+	char *filename_for_display, *uri, *escaped_filename;
 	GtkTreeRowReference *ref;
 	GFileMonitor *monitor;
 	GFile *file;
@@ -1704,13 +1706,16 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 		monitor = NULL;
 	}
 
+	escaped_filename = g_markup_escape_text (filename_for_display, -1);
 	gtk_list_store_insert_with_values (store, &iter, pos,
 			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
+			FILENAME_ESCAPED_COL, escaped_filename,
 			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
 			FILE_MONITOR_COL, monitor,
 			-1);
+	g_free (escaped_filename);
 
 	g_signal_emit (playlist,
 		       totem_playlist_table_signals[ITEM_ADDED],
@@ -1815,16 +1820,48 @@ totem_playlist_clear (TotemPlaylist *playlist)
 	return TRUE;
 }
 
+static int
+compare_removal (GtkTreeRowReference *ref, GtkTreePath *path)
+{
+	GtkTreePath *ref_path;
+	int ret = -1;
+
+	ref_path = gtk_tree_row_reference_get_path (ref);
+	if (gtk_tree_path_compare (path, ref_path) == 0)
+		ret = 0;
+	gtk_tree_path_free (ref_path);
+	return ret;
+}
+
+/* Whether the item in question will be removed */
+static gboolean
+totem_playlist_item_to_be_removed (TotemPlaylist *playlist,
+				   GtkTreePath *path,
+				   ClearComparisonFunc func)
+{
+	GList *ret;
+
+	if (func == NULL) {
+		GtkTreeSelection *selection;
+
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (playlist->priv->treeview));
+		return gtk_tree_selection_path_is_selected (selection, path);
+	}
+
+	ret = g_list_find_custom (playlist->priv->list, path, (GCompareFunc) compare_removal);
+	return (ret != NULL);
+}
+
 static void
 totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 				   ClearComparisonFunc func,
 				   gconstpointer data)
 {
 	GtkTreeRowReference *ref;
-	int next_pos;
+	GtkTreeRowReference *next;
 
 	ref = NULL;
-	next_pos = -1;
+	next = NULL;
 
 	if (func == NULL) {
 		GtkTreeSelection *selection;
@@ -1861,9 +1898,10 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 
 				path = gtk_tree_path_new_from_indices (i, -1);
 				r = gtk_tree_row_reference_new (playlist->priv->model, path);
-				if (playlist->priv->current != NULL) {
-					if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
+				if (playlist->priv->current_to_be_removed == FALSE && playlist->priv->current != NULL) {
+					if (gtk_tree_path_compare (path, playlist->priv->current) == 0) {
 						playlist->priv->current_to_be_removed = TRUE;
+					}
 				}
 				playlist->priv->list = g_list_prepend (playlist->priv->list, r);
 				gtk_tree_path_free (path);
@@ -1876,19 +1914,30 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 
 	/* If the current item is to change, we need to keep an static
 	 * reference to it, TreeIter and TreePath don't allow that */
-	if (playlist->priv->current != NULL) {
-		int *indices;
-
+	if (playlist->priv->current_to_be_removed == FALSE &&
+	    playlist->priv->current != NULL) {
 		ref = gtk_tree_row_reference_new (playlist->priv->model,
-				playlist->priv->current);
-		indices = gtk_tree_path_get_indices (playlist->priv->current);
-		next_pos = indices[0];
+						  playlist->priv->current);
+	} else if (playlist->priv->current != NULL) {
+		GtkTreePath *item;
 
+		item = gtk_tree_path_copy (playlist->priv->current);
+		gtk_tree_path_next (item);
+		next = gtk_tree_row_reference_new (playlist->priv->model, item);
+		while (next != NULL) {
+			if (totem_playlist_item_to_be_removed (playlist, item, func) == FALSE) {
+				/* Found the item after the current one that
+				 * won't be removed, thus the new current */
+				break;
+			}
+			gtk_tree_row_reference_free (next);
+			gtk_tree_path_next (item);
+			next = gtk_tree_row_reference_new (playlist->priv->model, item);
+		}
 	}
 
 	/* We destroy the items, one-by-one from the list built above */
-	while (playlist->priv->list != NULL)
-	{
+	while (playlist->priv->list != NULL) {
 		GtkTreePath *path;
 		GtkTreeIter iter;
 
@@ -1910,21 +1959,9 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 
 	if (playlist->priv->current_to_be_removed != FALSE) {
 		/* The current item was removed from the playlist */
-		if (next_pos != -1) {
-			char *str;
-			GtkTreeIter iter;
-			GtkTreePath *cur;
-
-			str = g_strdup_printf ("%d", next_pos);
-			cur = gtk_tree_path_new_from_string (str);
-
-			if (gtk_tree_model_get_iter (playlist->priv->model, &iter, cur) == FALSE) {
-				playlist->priv->current = NULL;
-				gtk_tree_path_free (cur);
-			} else {
-				playlist->priv->current = cur;
-			}
-			g_free (str);
+		if (next != NULL) {
+			playlist->priv->current = gtk_tree_row_reference_get_path (next);
+			gtk_tree_row_reference_free (next);
 		} else {
 			playlist->priv->current = NULL;
 		}
@@ -1939,9 +1976,7 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 	} else {
 		if (ref != NULL) {
 			/* The path to the current item changed */
-			playlist->priv->current =
-				gtk_tree_row_reference_get_path (ref);
-			gtk_tree_row_reference_free (ref);
+			playlist->priv->current = gtk_tree_row_reference_get_path (ref);
 		}
 
 		if (playlist->priv->shuffle)
@@ -1951,6 +1986,8 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 				totem_playlist_table_signals[CHANGED], 0,
 				NULL);
 	}
+	if (ref != NULL)
+		gtk_tree_row_reference_free (ref);
 	totem_playlist_update_save_button (playlist);
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->priv->treeview));
 
@@ -2139,6 +2176,7 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gboolean custom_title;
+	char *escaped_title;
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), FALSE);
 
@@ -2161,10 +2199,13 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 			return TRUE;
 	}
 
+	escaped_title = g_markup_escape_text (title, -1);
 	gtk_list_store_set (store, &iter,
 			FILENAME_COL, title,
+			FILENAME_ESCAPED_COL, escaped_title,
 			TITLE_CUSTOM_COL, TRUE,
 			-1);
+	g_free (escaped_title);
 
 	g_signal_emit (playlist,
 		       totem_playlist_table_signals[ACTIVE_NAME_CHANGED], 0);
@@ -2283,24 +2324,21 @@ totem_playlist_set_next (TotemPlaylist *playlist)
 
 	g_return_if_fail (TOTEM_IS_PLAYLIST (playlist));
 
-	if (totem_playlist_has_next_mrl (playlist) == FALSE)
-	{
+	if (totem_playlist_has_next_mrl (playlist) == FALSE) {
 		totem_playlist_set_at_start (playlist);
 		return;
 	}
 
 	totem_playlist_unset_playing (playlist);
 
-	if (playlist->priv->shuffle == FALSE)
-	{
+	if (playlist->priv->shuffle == FALSE) {
 		gtk_tree_model_get_iter (playlist->priv->model,
-				&iter,
-				playlist->priv->current);
+					 &iter,
+					 playlist->priv->current);
 
 		gtk_tree_model_iter_next (playlist->priv->model, &iter);
 		gtk_tree_path_free (playlist->priv->current);
-		playlist->priv->current = gtk_tree_model_get_path
-			(playlist->priv->model, &iter);
+		playlist->priv->current = gtk_tree_model_get_path (playlist->priv->model, &iter);
 	} else {
 		int indice;
 
